#include "card.h"

Card::Card(const QString _name, const QString _cost, const QString _colors, const QStringList _types, const QString _edition, const QString _rarity, const QString _text, const QString _flavor,  const QString _power, const QString _life) :
name(_name),cost (_cost), edition(_edition),text (_text),flavor (_flavor),colors(_colors), rarity(_rarity), types(_types), power(_power), life(_life), original_text(_text),original_cost(_cost), converted_cost{0}
{
    init_done = false;
    calculateConvertedCost();
    createSymbolsFromText();
    replaceFormatChars();
    createTableItems();
    type_string = types.size() >= 1 ? types.first() : "";
    init_done = true;
}

Card::Card(const QString _name, const QString _cost, const QString _colors, const QStringList _types, const QString _edition, const QString _rarity, const QString _text, const QString _flavor, const QString _power, const QString _life, short _standard, short _modern, short _commander) : Card(_name, _cost, _colors, _types, _edition, _rarity, _text, _flavor, _power, _life)
{
    legal_commander = _commander;
    legal_modern = _modern;
    legal_standard = _standard;
}

bool Card::operator ==(const QString& _name) const
{
    return name == _name;
}

void Card::createTableItems()
{
    /*
    tableItems.append(new QTableWidgetItem(name));
    tableItems.append(new QTableWidgetItem(cost));
    tableItems.append(new QTableWidgetItem(type_string));
    tableItems.append(new QTableWidgetItem(colors));
    tableItems.append(new QTableWidgetItem(power));
    tableItems.append(new QTableWidgetItem(life));
    tableItems.append(new QTableWidgetItem(rarity));
    tableItems.append(new QTableWidgetItem(edition));
    tableItems.append(new QTableWidgetItem(text));
    tableItems.append(new QTableWidgetItem(flavor));
    */
}

QVector<QTableWidgetItem *> Card::getTableItems() const
{
    QVector<QTableWidgetItem*> workaroundTableItems;


    workaroundTableItems.append(new QTableWidgetItem(name));
    workaroundTableItems.append(new QTableWidgetItem(cost));
    workaroundTableItems.append(new QTableWidgetItem(type_string));
    workaroundTableItems.append(new QTableWidgetItem(colors));
    workaroundTableItems.append(new QTableWidgetItem(power));
    workaroundTableItems.append(new QTableWidgetItem(life));
    workaroundTableItems.append(new QTableWidgetItem(rarity));
    workaroundTableItems.append(new QTableWidgetItem(edition));
    workaroundTableItems.append(new QTableWidgetItem(text));
    workaroundTableItems.append(new QTableWidgetItem(flavor));
    workaroundTableItems.append(new QTableWidgetItem());
    workaroundTableItems.last()->setData(Qt::EditRole, converted_cost);


    return workaroundTableItems;
}

void Card::createSymbolsFromText()
{
    if(init_done)
    {
        qDebug() << "createSymbolsFromText" << "Error calling after Init Done";
        return;
    }


    text.replace("%T", "↺");
    text.replace("%0", "⓪");
    text.replace("%1", "①");
    text.replace("%2", "②");
    text.replace("%3", "③");
    text.replace("%4", "④");
    text.replace("%5", "⑤");
    text.replace("%6", "⑥");
    text.replace("%7", "⑦");
    text.replace("%8", "⑧");
    text.replace("%9", "⑨");
    text.replace("%X", "Ⓧ");
    text.replace("%Y", "Ⓨ");
    text.replace("%Z", "Ⓩ");

    text.replace("%R", "🔥");
    text.replace("%W", "🔆");
    text.replace("%U", "💧"); //💧
    text.replace("%B", "💀");
    text.replace("%G", "🌳");
    text.replace("%C", "①");
    text.replace("%S", "(🌳|💧) ");
    text.replace("%A", "(🌳|🔆) ");
    text.replace("%H", "(②|💀) ");
    text.replace("%E", "(②|🔆) ");
    text.replace("%F", "(②|💧) ");
    text.replace("%J", "(②|🔥) ");
    text.replace("%M", "(②|🌳) ");

    text.replace("%D", "(🔆|💧) ");
    text.replace("%V", "(💧|💀) ");
    text.replace("%Q", "(💀|🌳) ");
    text.replace("%O", "(🔆|💀) ");
    text.replace("%P", "(🔥|🔆) ");
    text.replace("%L", "(🔥|🌳) ");
    text.replace("%I", "(💧|🔥) ");
    text.replace("%K", "(💀|🔥) ");
    text.replace("%C", "①");
    text.replace("%!", "(🔆|2L) ");
    text.replace("%`", "(💧|2L) ");
    text.replace("%@", "(🌳|2L) ");
    text.replace("%$", "(💀|2L) ");
    text.replace("%^", "(🔥|2L) ");


    cost.replace("R/P", "Ⓡ");
    cost.replace("G/P", "Ⓖ");
    cost.replace("W/P", "Ⓦ");
    cost.replace("B/P", "Ⓑ");
    cost.replace("U/P", "Ⓤ");
    cost.replace("0", "⓪");
    cost.replace("1", "①");
    cost.replace("2", "②");
    cost.replace("3", "③");
    cost.replace("4", "④");
    cost.replace("5", "⑤");
    cost.replace("6", "⑥");
    cost.replace("7", "⑦");
    cost.replace("8", "⑧");
    cost.replace("9", "⑨");
    cost.replace("X", "Ⓧ");
    cost.replace("R", "🔥");
    cost.replace("W", "🔆");
    cost.replace("U", "💧");
    cost.replace("B", "💀");
    cost.replace("G", "🌳");
    cost.replace("%A", "(🌳|🔆) ");
    cost.replace("%S", "(🌳|💧) ");
    cost.replace("%H", "(②|💀) ");
    cost.replace("%E", "(②|🔆) ");
    cost.replace("%F", "(②|💧) ");
    cost.replace("%J", "(②|🔥) ");
    cost.replace("%M", "(②|🌳) ");
    cost.replace("%D", "(🔆|💧) ");
    cost.replace("%V", "(💧|💀) ");
    cost.replace("%Q", "(💀|🌳) ");
    cost.replace("%O", "(🔆|💀) ");
    cost.replace("%P", "(🔥|🔆) ");
    cost.replace("%L", "(🔥|🌳) ");
    cost.replace("%I", "(💧|🔥) ");
    cost.replace("%K", "(💀|🔥) ");
    cost.replace("%C", "①"); //%E%F(②|💀) %J%M
    cost.replace("%!", "(🔆|2L) ");
    cost.replace("%`", "(💧|2L) ");
    cost.replace("%@", "(🌳|2L) ");
    cost.replace("%$", "(💀|2L) ");
    cost.replace("%^", "(🔥|2L) ");
    cost.replace("Y", "Ⓨ");
    cost.replace("Z", "Ⓩ");





}

void Card::calculateConvertedCost()
{
    if(init_done)
    {
        qDebug() << "calculateConvertedCost" << "Error calling after Init Done";
        return;
    }



    if(original_cost.isEmpty())
    {
        converted_cost=0;
    }

    else
    {
        if(original_cost.contains("/"))
        {

             QStringList parts = original_cost.split("/");
             if(parts.size() > 1)
             {
                 converted_cost = std::max(calculateConvertedCost(parts[0]), calculateConvertedCost(parts[1]) );
             }
             else if (parts.size() == 1)
             {
                 converted_cost = calculateConvertedCost(parts[0]);
             }
             else
             {
                 qDebug() << "Buggy costs in calculateConvertedCost" << parts << " ||| " << *this;
             }
             //qDebug() << original_cost << " " << converted_cost << parts;

        }
        converted_cost = calculateConvertedCost(original_cost);
    }

}

int Card::calculateConvertedCost(const QString &_text)
{
    if(init_done)
    {
        qDebug() << "calculateConvertedCost" << "Error calling after Init Done";
        return 0;
    }


    if(_text.isEmpty())
    {
        return 0;
    }

    int a = 0;
    QString numbers;
    int result = 0;
    while(_text[a].isDigit() || _text[a] == "X")
    {
        if(_text[a] == "X")
        {
            a++;
            if(!(a < _text.size()))
            {
                break;
            }
            continue;
        }
        numbers+=_text[a];
        a++;
        if(!(a < _text.size()))
        {
            break;
        }
    }

    result += numbers.toInt();
    if(_text.contains("%"))
    {
        result += _text.count("%");
    }


    for(int i = a; i < _text.size(); i++)
    {
        if(_text[a].isLetter())
        {
            result++;
        }
    }
    return result;
}

void Card::replaceFormatChars()
{
    if(init_done)
    {
        qDebug() << "replaceFormatChars" << "Error calling after Init Done";
        return;
    }

    text.replace('\n', " ");
    text.replace('#', " ");
}

Card::Card()
{
    qDebug() << "Error calling Empty Constructor of Card";

}

Card::operator QString() const
{
    QString type_string;

    for(int i = 0; i < types.size(); i++)
    {
        type_string += types[i];
    }

    return name + " | " + cost + " | " + type_string + " | " +  edition + " | (" +  rarity + ") | " + text + " | " +  flavor + " | " + colors  + " |  " + power + " | " + life + "#";
}
