#include "mainwindow.h"
#include "ui_mainwindow.h"

int savecounter = 0;
const int NUMBER_OF_BACKUP_DECKS = 5;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    connectSlots();
    finetuneUi();
    //loadCards();
    loadJSonCards();
    renderCards();
    ui->tableWidget->setSortingEnabled(true);
    loadEditions();
    updateCardStatistic();

	QPixmap icon;
	icon.loadFromData(QByteArray::fromBase64(MainWindow::iconb64));
	this->setWindowIcon(QIcon(icon));


}

void MainWindow::loadJSonCards()
{
    //GET https://api.scryfall.com/bulk-data

    QFile loadFile("cards.json");

    if (!loadFile.open(QIODevice::ReadOnly))
    {
        qWarning("Couldn't open save file.");
        return;
    }

    QByteArray saveData = loadFile.readAll();

    const QJsonDocument loadDoc(QJsonDocument::fromJson(saveData));
    const QJsonArray cardarray (loadDoc.array());


    int number_of_threads = -1;

    #pragma omp parallel
    {
        number_of_threads = omp_get_num_threads();
    }

    std::vector<Card> tempcards[number_of_threads];



    #pragma omp parallel for
	for(int i = 0; i < cardarray.size(); i++)
    {
        const QString name = cardarray[i].toObject()["name"].toString();
        const QString edition = cardarray[i].toObject()["set"].toString();
        QString rarity = cardarray[i].toObject()["rarity"].toString();

        rarity = rarity == "uncommon" ? "U" : rarity;
        rarity = rarity == "rare" ? "R" : rarity;
        rarity = rarity == "common" ? "C" : rarity;
        rarity = rarity == "mythic" ? "M" : rarity;
        rarity = rarity == "special" ? "S" : rarity;
        rarity = rarity == "bonus" ? "B" : rarity;


        QString colors = cardarray[i].toObject()["colors"].toArray().size() > 1 ? "Gld" : cardarray[i].toObject()["colors"].toArray()[0].toString();
        const QString costs = cardarray[i].toObject()["mana_cost"].toString().remove("{").remove("}");
        QString type = cardarray[i].toObject()["type_line"].toString();

        colors = type.contains("Land") == true ? colors + " Lnd" : colors;
        colors = colors.isEmpty() && type.contains("Artifact") ? "Art" : colors;

        //QRegularExpression("/[A-Z]}
        QString text = cardarray[i].toObject()["oracle_text"].toString();

        text = text.replace("{R/P}", "Ⓡ");
        text = text.replace("{G/P}", "Ⓖ");
        text = text.replace("{W/P}", "Ⓦ");
        text = text.replace("{B/P}", "Ⓑ");
        text = text.replace("{U/P}", "Ⓤ");

        while(text.indexOf(QRegularExpression("/[A-Z]}")) > -1)
        {
            int i = text.indexOf(QRegularExpression("/[A-Z]}"));
			text[i] = 'ß';
        }


		text = text.replace("ß", "/%");
        text = text.replace("{Q}", "↻");
        text = text.replace("{T}", "↺");
        text = text.replace("{CHAOS}", "☯");
        text = text.replace("{HW}", "🤡");
        text = text.replace("{A}", "🤡");
        text = text.replace("{HR}", "🤡");
        text = text.replace("{E}", "⚡");
        text = text.replace("{P}", "⏀");
        text = text.replace("{S}", "❄");


        text.replace("{", "%").replace(QRegularExpression("/[A-Z]}"),"/%[A-Z]").remove("}");
        const QString flavor = cardarray[i].toObject()["flavor_text"].toString();

        //,"power":"5","toughness":"3"

        QString power = cardarray[i].toObject().contains("power")  ? cardarray[i].toObject()["power"].toString() : "";
        QString life = cardarray[i].toObject().contains("toughness")  ? cardarray[i].toObject()["toughness"].toString() : "";

        short legal_commander = (cardarray[i].toObject()["legalities"].toObject()["commander"].toString() == "legal") ? 1 : 0;
        short legal_modern = (cardarray[i].toObject()["legalities"].toObject()["modern"].toString() == "legal") ? 1 : 0;
        short legal_standard = (cardarray[i].toObject()["legalities"].toObject()["standard"].toString() == "legal") ? 1 : 0;

        //tempcards[omp_get_thread_num()].push_back(Card(name,costs, colors, QStringList(type),edition,rarity,text,flavor,power,life));
        tempcards[omp_get_thread_num()].push_back(Card(name,costs, colors, QStringList(type),edition,rarity,text,flavor,power,life,legal_standard,legal_modern,legal_commander));
    }

    for(int i = 0; i < number_of_threads; i++)
    {
        cards.insert(cards.end(), tempcards[i].begin(), tempcards[i].end());
    }
}

void MainWindow::loadCards()
{
    QFile file("cleanfile.csv");
    if (!file.open(QIODevice::ReadOnly))
    {
        qDebug() << file.errorString();
        return;
    }

    QString all = file.readAll();
    file.close();
    //sanitzeData(all);
    QStringList properties = tokenize(all);

    for(int i = 9; i < properties.size(); i = i+9)
    {
        const QString& name = properties[i+0];
        const QString& edition = properties[i+1];
        const QString& rarity = properties[i+2];
        const QString& colors = properties[i+3];
        const QString& costs = properties[i+4];
        const QString& power_and_life = properties[i+5];
        const QString& type = properties[i+6];
        const QString& text = properties[i+7];
        const QString& flavor = properties[i+8];

        QString power = power_and_life.isEmpty() ? "" : power_and_life.split("/").first();
        QString life = power_and_life.isEmpty() ? "" : power_and_life.split("/").at(1);

        if(power_and_life.contains("|"))
        {
            power = power_and_life.split("|").at(0);
            life = power_and_life.split("|").at(1);
        }

        cards.push_back(Card(name,costs, colors, QStringList(type),edition,rarity,text,flavor,power,life));
    }
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::renderCards()
{
    if(render_is_running == true)
    {
        return;
    }

    render_is_running = true;
    //this->setEnabled(false);
    disconnectTableSlots();
    resetCardTable();
    //ui->tableWidget->clearContents();
    //ui->tableWidget->setRowCount(0);
    int counter = 0;


    ui->tableWidget->setSortingEnabled(false);

    ui->tableWidget->hide();

	for(uint i = 0; i < cards.size(); i++)
    {
        if(!isCardFiltered(i))
        {
            const int current_row = ui->tableWidget->rowCount();
            ui->tableWidget->insertRow(current_row);

            QVector<QTableWidgetItem*> items = cards[i].getTableItems();

            if(items.size() != 11)
            {
                qDebug() <<  "items.size()" << items.size();
                qDebug() << "cards[i]" << cards[i];
                qDebug() << "items" << items;
            }
            for(int x = 0; x < items.size(); x++)
            {
                if(items[x] == nullptr)
                {
                    qDebug() << "Card tableItem is nullpointer " << cards[i] << "//" << items.size() << "//" << items;
                }
                ui->tableWidget->setItem(current_row, x, items[x]);
                //ui->tableWidget->setCellWidget(i,x, new QLabel(cards[i].text));
            }
            counter++;
        }
    }

    ui->tableWidget->show();
    ui->tableWidget->setSortingEnabled(true);


    connectTableSlots();
    render_is_running = false;

    //this->setEnabled(true);

    qDebug() << "Cards which should be displayed: " << counter;
}

void MainWindow::renderCardsParallel()
{
    disconnectTableSlots();
    resetCardTable();
    ui->tableWidget->setSortingEnabled(false);
    //ui->tableWidget->clearContents();
    //ui->tableWidget->setRowCount(0);
    int counter = 0;

    ui->tableWidget->hide();

    const int number_of_threads = omp_get_max_threads();
    QVector<QVector <QTableWidgetItem*> > temp_items [number_of_threads];
    qDebug() << "Number of threads " << number_of_threads;

    #pragma omp parallel for
	for(uint i = 0; i < cards.size(); i++)
    {
        if(!isCardFiltered(i))
        {
            int id = omp_get_thread_num();
            temp_items[id].push_back(cards[i].getTableItems());
        }
    }

    const int items_size = cards[0].getTableItems().size();

    for(int a = 0; a < number_of_threads; a++)
    {
        for(int b = 0; b < temp_items[a].size(); b++)
        {
            int current_row = ui->tableWidget->rowCount();
            ui->tableWidget->insertRow(current_row);

            for(int x = 0; x < items_size; x++)
            {
                ui->tableWidget->setItem(current_row, x, temp_items[a][b][x]);
            //ui->tableWidget->setCellWidget(i,x, new QLabel(cards[i].text));
            }
            counter++;
        }
    }




        /*
        else
        {
            qDebug() << cards[i].get_name() << "(" << cards[i].get_colors() << ")" << " is filtered";
        }
        */


    ui->tableWidget->show();

    connectTableSlots();
    ui->tableWidget->setSortingEnabled(true);

    qDebug() << "Cards which should be displayed: " << counter;
}

QStringList MainWindow::tokenize(QString &_data)
{
    enum
    {
        string_open = 1,
        string_closed = 2
    };
    int state = string_closed;
    QString temp = "";
    QStringList properties;

    for(int i = 0; i < _data.size(); i++)
    {
        if(state == string_closed && _data[i] == '\"')
        {
            state = string_open;
            if(_data[i+1] == '\"' && _data[i+2] == '\"')
            {
                _data[i+1] = ' ';
                _data[i+2] = ' ';
            }
        }
        else if (state == string_open &&  _data[i] == '\"')
        {
            if(_data[i+1] == '\"' && _data[i+2] == '\"')
            {
                _data[i+1] = ' ';
                _data[i+2] = ' ';
            }

            properties.push_back(temp);
            temp = "";
            state = string_closed;
        }

        else if(state == string_open)
        {
            temp += _data[i];
        }
    }
    return properties;
}

void MainWindow::sanitzeData(QString &_data, int _counter)
{
    _data.replace("\"\"\"", "\"");
    for(int i = 1; i < _data.size()-2; i++)
    {
        if( _data[i] == '"' && _data[i+1] == '"' )
        {
            if ( (_data[i-1] != ';' && _data[i-1] != '\n') || (_data[i+2] != ';' && _data[i+2] != '\n') )
            {
                _data[i] = ' ';
            }
        }
        else if( _data[i] == '"' && _data[i+1] == '\n' && _data[i+2] != '"')
        {
            _data[i] = ' ';
        }
    }

    _data.replace(" \"", "");
    if(_counter < 2)
    {
        sanitzeData(_data,_counter+1);
    }
}

void MainWindow::debugOutput()
{
    //sanitzeData(all);

    /*
    // first, we open the file
    QFile outfile("outfile.txt");
    outfile.open(QIODevice::WriteOnly);

    // now that we have a file that allows us to write anything to it,
    // we need an easy way to write out text to it
    QTextStream qout(&outfile);

    for(int i = 0; i < properties.size(); i++)
    {
        qout << i << ". " << properties[i] << "\n";
        if( (i+1) % 9 == 0 && i > 0)
        {
            qout << "=============================" << "\n";
        }
    }



    // and when you're done, make sure to close the file
    outfile.close();


    // first, we open the file
    QFile cleanfile("cleanfile.csv");
    cleanfile.open(QIODevice::WriteOnly);

    // now that we have a file that allows us to write anything to it,
    // we need an easy way to write out text to it
    QTextStream clout(&cleanfile);

    clout << all;
    // and when you're done, make sure to close the file
    cleanfile.close();
    */
}

void MainWindow::finetuneUi()
{
    ui->tableWidget->setColumnWidth(1, ui_const_width_cost);
    ui->tableWidget->setColumnWidth(0,ui_const_width_name);
    ui->tableWidget->setColumnWidth(8,ui_const_width_text);
    ui->tableWidget->setColumnWidth(2,ui_const_width_type);

    ui->tableWidget->setColumnWidth(4,ui_const_width_power);
    ui->tableWidget->setColumnWidth(5,ui_const_width_life);
    ui->tableWidget->setColumnWidth(6,ui_const_width_rarity);

    ui->decktable->setColumnWidth(1+1, ui_const_width_cost);
    ui->decktable->setColumnWidth(1+0,ui_const_width_name);
    ui->decktable->setColumnWidth(1+8,ui_const_width_text);
    ui->decktable->setColumnWidth(1+2,ui_const_width_type);

    ui->decktable->setColumnWidth(1+4,ui_const_width_power);
    ui->decktable->setColumnWidth(1+5,ui_const_width_life);
    ui->decktable->setColumnWidth(1+6,ui_const_width_rarity);

    ui->decktable->setColumnWidth(1+6,ui_const_width_rarity);

    ui->decktable->setColumnWidth(0,ui_const_width_quantity);
    //ui->decktable->setColumnWidth(0,ui_const_width_flavor);

    /*
    ui_const_width_rarity
    ui_const_width_power
    ui_const_width_life
    */
}

void MainWindow::loadDeck(const QString &_name)
{
    QFile file(_name);
    if (!file.open(QIODevice::ReadOnly)) {
        qDebug() << file.errorString();
        return;
    }

    int counter = 0;
    while (!file.atEnd())
    {
        ui->decktable->insertRow(counter);
        for(int i = 0; i < ui->decktable->columnCount(); i++)
        {
            QString line = file.readLine();
            QByteArray data = QByteArray::fromBase64(line.toStdString().c_str());
            QString item(data.toStdString().c_str());
            //qDebug() << item;

            ui->decktable->setItem(counter,i, new QTableWidgetItem(item));
        }
        //qDebug() << "====================================";
        counter++;
    }
    file.close();
}

void MainWindow::saveDeck(const QString &_name)
{
    QFile outfile(_name);
    outfile.open(QIODevice::WriteOnly);

    // now that we have a file that allows us to write anything to it,
    // we need an easy way to write out text to it
    QTextStream qout(&outfile);

    for(int i = 0; i < ui->decktable->rowCount(); i++)
    {
        QStringList rowdata = serializeRow(i);
		//qDebug() << rowdata;
        for(int a = 0; a < rowdata.size(); a++)
        {
            qout << rowdata[a].toUtf8().toBase64() << "\n";
        }
    }
    outfile.close();
}

QStringList MainWindow::serializeRow(int _row)
{
    QStringList data;
    for(int i = 0; i < ui->decktable->columnCount(); i++)
    {
        data.push_back(ui->decktable->item(_row,i)->text());
    }
    return data;
}

void MainWindow::connectSlots()
{
    ui->tableWidget->setContextMenuPolicy(Qt::CustomContextMenu);
    ui->decktable->setContextMenuPolicy(Qt::CustomContextMenu);


    //connect(ui->tableWidget, SIGNAL(cellDoubleClicked(int,int)), this, SLOT(addCardToDeck(int,int)));
    connect(ui->tableWidget, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(addCardToDeck(QPoint)));
    connect(ui->decktable, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(removeCardFromDeck(QPoint)));
    connect(ui->tableWidget, SIGNAL(cellClicked(int,int)) , this, SLOT(showCardDescription(int,int)));
    connect(ui->button_add,SIGNAL(pressed()),this,SLOT(addCardButton()));
    connect(ui->button_delete,SIGNAL(pressed()),this,SLOT(removeCardButton()));
    connect(ui->button_load,SIGNAL(pressed()),this,SLOT(loadButton()));
    connect(ui->button_save,SIGNAL(pressed()),this,SLOT(saveButton()));

    connect(ui->filter_color_artifact, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_color_red, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_color_green, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_color_black, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_color_white, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_color_blue, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_color_gold, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_color_land, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));

    connect(ui->filter_rarety_common, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_rarety_rare, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_rarety_uncommon, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_rarety_mythic, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_rarety_special, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_rarety_bonus, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));

    connect(ui->filter_creature, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_spell_enchament, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_spell_instant, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_spell_sorcery, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_artifact, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_planeswalker, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));
    connect(ui->filter_other_type, SIGNAL(stateChanged(int)), this,SLOT(checkFilters()));

    connect(ui->search1, SIGNAL(editingFinished()),this, SLOT(handleSearch()));
    connect(ui->search2, SIGNAL(editingFinished()),this, SLOT(handleSearch()));
    connect(ui->search3, SIGNAL(editingFinished()),this, SLOT(handleSearch()));
    connect(ui->search4, SIGNAL(editingFinished()),this, SLOT(handleSearch()));
    connect(ui->search_inverse1, SIGNAL(editingFinished()),this, SLOT(handleSearch()));
    connect(ui->search_inverse2, SIGNAL(editingFinished()),this, SLOT(handleSearch()));

    connect(ui->actionExport_Deck, SIGNAL(triggered(bool)),this, SLOT(exportDeck()));
    connect(ui->filter_format, SIGNAL(currentIndexChanged(int)), this, SLOT(handleSearch()));
    connect(ui->filter_power_min, SIGNAL(textEdited(QString)), this, SLOT(handleSearch()));
    connect(ui->filter_power_max, SIGNAL(textEdited(QString)), this, SLOT(handleSearch()));
    connect(ui->filter_life_min, SIGNAL(textEdited(QString)), this, SLOT(handleSearch()));
    connect(ui->filter_life_max, SIGNAL(textEdited(QString)), this, SLOT(handleSearch()));
    connect(ui->filter_costs_min, SIGNAL(textEdited(QString)), this, SLOT(handleSearch()));
    connect(ui->filter_costs_max, SIGNAL(textEdited(QString)), this, SLOT(handleSearch()));

    connect(ui->actionReload,SIGNAL(triggered(bool)),this,SLOT(reloadCards()));
    connect(ui->actionNew_Deck,SIGNAL(triggered(bool)),this,SLOT(createNewDeck()));

    connect(ui->filter_multicolored, SIGNAL(stateChanged(int)), this, SLOT(handleSearch() ));

    connect(ui->actionExport_to_MWS_Format, SIGNAL(triggered(bool)),this, SLOT(exportDeckMWS()));

    connect(ui->actionPrint_Proxies, SIGNAL(triggered(bool)), this,SLOT(showProxyExport()));

    connect(ui->actionStart_Decktester, SIGNAL(triggered(bool)),this,SLOT(testDeck()));
}

void MainWindow::disconnectTableSlots()
{

    disconnect(ui->tableWidget, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(addCardToDeck(QPoint)));
    disconnect(ui->tableWidget, SIGNAL(cellClicked(int,int)) , this, SLOT(showCardDescription(int,int)));

}

void MainWindow::connectTableSlots()
{

    connect(ui->tableWidget, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(addCardToDeck(QPoint)));
    connect(ui->tableWidget, SIGNAL(cellClicked(int,int)) , this, SLOT(showCardDescription(int,int)));

}

void MainWindow::resetCardTable()
{
    /*
    ui->tableWidget->clear();
    ui->tableWidget->setRowCount(0);
    ui->tableWidget->setHorizontalHeaderItem(0, new QTableWidgetItem("Name"));
    ui->tableWidget->setHorizontalHeaderItem(1, new QTableWidgetItem("Cost"));
    ui->tableWidget->setHorizontalHeaderItem(2, new QTableWidgetItem("Type"));
    ui->tableWidget->setHorizontalHeaderItem(3, new QTableWidgetItem("Color"));
    ui->tableWidget->setHorizontalHeaderItem(4, new QTableWidgetItem("Pow."));
    ui->tableWidget->setHorizontalHeaderItem(5, new QTableWidgetItem("Life"));
    ui->tableWidget->setHorizontalHeaderItem(6, new QTableWidgetItem("Rarit."));
    ui->tableWidget->setHorizontalHeaderItem(7, new QTableWidgetItem("Edition"));
    ui->tableWidget->setHorizontalHeaderItem(8, new QTableWidgetItem("Text"));
    ui->tableWidget->setHorizontalHeaderItem(9, new QTableWidgetItem("Flavor"));
    ui->tableWidget->setHorizontalHeaderItem(10, new QTableWidgetItem("Conv."));
    */
    ui->tableWidget->clearContents();
    ui->tableWidget->model()->removeRows(0, ui->tableWidget->rowCount());
}

void MainWindow::toggleShowOnlyGoldCardsWithSelectedColor()
{
    limit_gold_to_selected_colors = !limit_gold_to_selected_colors;
    renderCards();
}

void MainWindow::debugGetAllEditions()
{
    QStringList editions;


	for(uint i = 0; i < cards.size(); i++)
    {
        QString edition = cards[i].get_edition();
        if(!editions.contains(edition))
        {
            editions.push_back(edition);
        }
    }
    //qDebug() << editions;
    debugGetACardFromEachEdition(editions);
}

void MainWindow::debugGetACardFromEachEdition(const QStringList& _editions)
{
    for(int a = 0; a < _editions.size(); a++)
    {
		for(uint b = 0; b < cards.size(); b++)
        {
            if(cards[b].get_rarity()=="R" && cards[b].get_edition() == _editions[a])
            {
                qDebug() << cards[b].get_edition() << " " << cards[b].get_name();
                break;
            }

        }
    }
}

int MainWindow::cardNameToIndex(const QString &_name, const QString &_edition)
{
	for(uint i = 0; i < cards.size();i++)
    {
        if(cards[i].get_name()==_name && cards[i].get_edition() == _edition)
        {
            return i;
        }
    }
    return -1;
}

void MainWindow::loadEditions()
{
    QFile file("editions.csv");
    if (!file.open(QIODevice::ReadOnly)) {
        qDebug() << file.errorString();
        return;
    }

    QMap<QString,QString> long2short;
    while (!file.atEnd())
    {
        QString line = file.readLine();
        QString longname =line.split(";").first();
        QString shortname =line.split(";")[1];

        long2short.insert(longname.toUpper(),shortname.toUpper().replace("\n", ""));
    }
    file.close();

    QFile file2("modern.txt");
    if (!file2.open(QIODevice::ReadOnly)) {
        qDebug() << file2.errorString();
        return;
    }
    while (!file2.atEnd())
    {
        this->format_modern.append(file2.readLine().toUpper().replace("\n", ""));

    }
    file2.close();

    QFile file3("standard.txt");
    if (!file3.open(QIODevice::ReadOnly)) {
        qDebug() << file3.errorString();
        return;
    }
    while (!file3.atEnd())
    {
        this->format_standard.append(file3.readLine().replace("\n", ""));

    }
    file3.close();


    for(int i = 0; i < format_modern.size(); i++)
    {
        if(long2short.contains(format_modern[i]))
        {
            format_modern[i] = long2short[format_modern[i]];
        }
        else
        {
            qDebug() << format_modern[i] << " is not in list";
        }
    }

    //qDebug() << format_modern;


    for(int i = 0; i < format_standard.size(); i++)
    {
        if(long2short.contains(format_standard[i]))
        {
            format_standard[i] = long2short[format_standard[i]];
        }

        else
        {
            qDebug() << format_standard[i] << " is not in list";
        }

    }

    //qDebug() << edition_standard;
}

void MainWindow::changeView()
{
    qDebug() << "changeView";
    ui->tableWidget->clear();
}

void MainWindow::addCardToDeck(int _row, int _col)
{
    savecounter = (savecounter + 1) % NUMBER_OF_BACKUP_DECKS;
	_col++; //DEBUG REMOVE


    this->saveDeck(QString("backup_file") + QString::number(savecounter)  +  ".deck");

	const QTableWidgetItem* cardname_item = ui->tableWidget->item(_row,0);

	ui->decktable->setSortingEnabled(false);


	if(cardname_item == NULL)
    {
        qDebug() << "ui->tableWidget->item("<<_row<<",0)" << " is a nullpointer";
        return;
    }

	QString cardname = cardname_item->text();

    int present_row = checkIfCardIsAlreadyPresent(cardname);

	QTableWidgetItem* quantity_item = NULL;

	if(present_row > -1)
	{
		quantity_item = ui->decktable->item(present_row,0);
		if(quantity_item == NULL)
		{
			qDebug() << "ERROR ui->decktable->item("<<present_row<<",0)";
			return;
		}
		else
		{
			quantity_item->setText(QString::number(quantity_item->text().toInt()+1));
		}
	}

    else
    {
		/*
		QMap<QString,QString> fields;
		fields["number"] = "1";

		const QTableWidgetItem* lib_item_name = ui->tableWidget->item(_row,0);
		if(!lib_item_name)
		{
			qDebug() << "ERROR lib_item_name is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_name is NULL addCardToDeck");
			return;
		}
		else if(lib_item_name->text().isEmpty())
		{
			qDebug() << "ERROR lib_item_name has no name addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_name has no name addCardToDeck");
			return;
		}
		fields["name"] = lib_item_name->text();


		const QTableWidgetItem* lib_item_cost = ui->tableWidget->item(_row,1);
		if(!lib_item_cost)
		{
			qDebug() << "ERROR lib_item_cost is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_cost is NULL addCardToDeck");
			return;
		}
		fields["cost"] = lib_item_cost->text();


		const QTableWidgetItem* lib_item_type = ui->tableWidget->item(_row,2);
		if(!lib_item_type)
		{
			qDebug() << "ERROR lib_item_type is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_type is NULL addCardToDeck");
			return;
		}
		else if(lib_item_type->text().isEmpty())
		{
			qDebug() << "ERROR lib_item_type has no name addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_type has no name addCardToDeck");
			return;
		}
		fields["type"] = lib_item_type->text();

		const QTableWidgetItem* lib_item_color = ui->tableWidget->item(_row,3);
		if(!lib_item_color)
		{
			qDebug() << "ERROR lib_item_color is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_color is NULL addCardToDeck");
			return;
		}
		fields["color"] = lib_item_color->text();

		const QTableWidgetItem* lib_item_power = ui->tableWidget->item(_row,4);
		if(!lib_item_power)
		{
			qDebug() << "ERROR lib_item_power is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_power is NULL addCardToDeck");
			return;
		}
		fields["power"] = lib_item_power->text();

		const QTableWidgetItem* lib_item_life = ui->tableWidget->item(_row,5);
		if(!lib_item_life)
		{
			qDebug() << "ERROR lib_item_life is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_life is NULL addCardToDeck");
			return;
		}
		fields["life"] = lib_item_life->text();

		const QTableWidgetItem* lib_item_rarity = ui->tableWidget->item(_row,6);
		if(!lib_item_rarity)
		{
			qDebug() << "ERROR lib_item_rarity is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_rarity is NULL addCardToDeck");
			return;
		}
		fields["rarity"] = lib_item_rarity->text();

		const QTableWidgetItem* lib_item_edition = ui->tableWidget->item(_row,7);
		if(!lib_item_edition)
		{
			qDebug() << "ERROR lib_item_edition is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_edition is NULL addCardToDeck");
			return;
		}
		fields["edition"] = lib_item_edition->text();

		const QTableWidgetItem* lib_item_description = ui->tableWidget->item(_row,8);
		if(!lib_item_description)
		{
			qDebug() << "ERROR lib_item_description is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_description is NULL addCardToDeck");
			return;
		}
		fields["description"] = lib_item_description->text();

		const QTableWidgetItem* lib_item_flavor = ui->tableWidget->item(_row,9);
		if(!lib_item_flavor)
		{
			qDebug() << "ERROR lib_item_flavor is NULL addCardToDeck";
			ui->menuCards_in_Deck->setTitle("ERROR lib_item_flavor is NULL addCardToDeck");
			return;
		}
		fields["flavor"] = lib_item_flavor->text();


		qDebug() << fields;
		qDebug() << ui->tableWidget->selectedItems();
		*/

		const int newrow =ui->decktable->rowCount();

        ui->decktable->insertRow(newrow);

		QTableWidgetItem* quantity = new QTableWidgetItem("1");
		ui->decktable->setItem(newrow,0,quantity);

        for(int i = 0; i < ui->decktable->columnCount(); i++)
        {
			const QTableWidgetItem* lib_item = ui->tableWidget->item(_row,i);

			if(lib_item == NULL)
			{
				qDebug() << "ERROR lib_item is NULL ui->tableWidget->item("<<present_row<<","<< i <<")";

			}

			QTableWidgetItem* newitem = new QTableWidgetItem(lib_item->text());
            ui->decktable->setItem(newrow, i+1, newitem);
        }
    }
	ui->decktable->setSortingEnabled(true);
    updateCardStatistic();
}

void MainWindow::addCardToDeck(const QPoint& _point)
{
    const QTableWidgetItem* temp =  ui->tableWidget->itemAt(_point);

    if(temp != nullptr)
    {
        addCardToDeck(temp->row(), temp->column());
    }
    else
    {
        qDebug() << "MainWindow::addCardToDeck(const QPoint& _point) is null";
    }

}

void MainWindow::removeCardFromDeck(const QPoint &_point)
{
    const QTableWidgetItem* temp =  ui->decktable->itemAt(_point);

	if(temp == NULL)
	{
		return;
	}

    if(ui->decktable->rowCount() > 0)
    {
        int row = temp->row();
        QString cardname = ui->decktable->item(row,1)->text();
        int present_row = checkIfCardIsAlreadyPresent(cardname);

        if(present_row > -1)
        {
            ui->decktable->item(present_row,0)->setText(QString::number(ui->decktable->item(present_row,0)->text().toInt()-1));

            if(ui->decktable->item(present_row,0)->text().toInt() < 1)
            {
                ui->decktable->removeRow(ui->decktable->currentRow());
            }
        }

        else
        {
            ui->decktable->removeRow(ui->decktable->currentRow());
        }
    }
    updateCardStatistic();
}

void MainWindow::addCardButton()
{
    if(ui->tableWidget->currentRow() > -1)
    {
        addCardToDeck(ui->tableWidget->currentRow(),0);
    }
    else
    {
        qDebug() << "Warning current row " << ui->tableWidget->currentRow();
    }
}

void MainWindow::removeCardButton()
{
    if(ui->decktable->rowCount() > 0 && ui->decktable->currentRow() > -1)
    {
        int row = ui->decktable->currentRow();
        QString cardname = ui->decktable->item(row,1)->text();
        int present_row = checkIfCardIsAlreadyPresent(cardname);

        if(present_row > -1)
        {
            ui->decktable->item(present_row,0)->setText(QString::number(ui->decktable->item(present_row,0)->text().toInt()-1));

            if(ui->decktable->item(present_row,0)->text().toInt() < 1)
            {
                ui->decktable->removeRow(ui->decktable->currentRow());
            }
        }

        else
        {
            ui->decktable->removeRow(ui->decktable->currentRow());
        }
    }
    else
    {
        qDebug() << "Warning Rowcount and current row " << ui->decktable->rowCount() << " " << ui->decktable->currentRow();
    }
    updateCardStatistic();
}

void MainWindow::saveButton()
{
    QString fileName = QFileDialog::getSaveFileName(this,tr("Save Deck"), lastLoadedDeck.isEmpty() ? "" : lastLoadedDeck, tr("Deck Files (*.deck)"));
    lastLoadedDeck = fileName;

    fileName = fileName.endsWith(".deck") ? fileName : fileName+".deck";
    saveDeck(fileName);
}

void MainWindow::loadButton()
{
    QString fileName = QFileDialog::getOpenFileName(this,tr("Open Deck"), lastLoadedDeck.isEmpty() ? "" : lastLoadedDeck, tr("Deck Files (*.deck)"));
    lastLoadedDeck = fileName;
    ui->decktable->setRowCount(0);
    loadDeck(fileName);
    updateCardStatistic();
}

void MainWindow::updateCardStatistic()
{
    ui->menuCards_in_Display->setTitle("Cards in Display: " + QString::number(ui->tableWidget->rowCount()) );

    int counter = 0;
    for(int i = 0; i < ui->decktable->rowCount();i++)
    {
		//counter += ui->decktable->item(i,0)->text().toInt();
		const QTableWidgetItem* item = ui->decktable->item(i,0);

		if(item != NULL)
		{
			counter += ui->decktable->item(i,0)->text().toInt();
		}
		else
		{
			qDebug("ERROR updateCardStatistic == NULL");
			ui->menuCards_in_Deck->setTitle("ERROR updateCardStatistic == NULL");
			return;
		}
    }

    ui->menuCards_in_Deck->setTitle("Cards in Deck: " + QString::number(counter) );
}

void MainWindow::reloadCards()
{
    ui->tableWidget->setSortingEnabled(false);
    cards.clear();
    //loadCards();
    loadJSonCards();
    renderCards();
    ui->tableWidget->setSortingEnabled(true);
}

void MainWindow::createNewDeck()
{
    ui->decktable->clearContents();
    ui->decktable->setRowCount(0);
    lastLoadedDeck = "";
}

int MainWindow::checkIfCardIsAlreadyPresent(const QString &_name)
{
    for(int i = 0; i < ui->decktable->rowCount();i++)
    {
		if(ui->decktable->item(i,1) == NULL)
        {
            qDebug() << "NULLPOINTER! int MainWindow::checkIfCardIsAlreadyPresent(const QString &_name): ui->decktable->item(i,1) == nullptr";
            exit(0);
            return -1;

        }

        else if(ui->decktable->item(i,1)->text() == _name)
        {
            return i;
        }
    }
    return -1;
}

bool MainWindow::isCardFiltered(int card_index)
{
    const QString color = cards[card_index].get_colors();
    const QString rarity = cards[card_index].get_rarity();
    const QString cost = cards[card_index].get_cost();
    const QString type = cards[card_index].get_types().size() > 0 ? cards[card_index].get_types().first() : "";

    return checkFilterColors(card_index) || checkFilterRarity(rarity ) || searchFilter(card_index) || checkFilterType(type,cost) || checkFilterFormat(card_index) || checkFilterPower(card_index) || checkFilterCosts(card_index) || checkFilterLife(card_index);
}

bool MainWindow::checkFilterColors(int _card_index)
{
    const QString _color = cards[_card_index].get_colors();

    if(_color != "Gld" && ui->filter_multicolored->isChecked())
    {
        return true;
    }

    if(_color == "U" && !ui->filter_color_blue->isChecked())
    {
        return true;
    }
    else if(_color == "R" && !ui->filter_color_red->isChecked())
    {
        return true;
    }
    else if(_color == "G" && !ui->filter_color_green->isChecked())
    {
        return true;
    }
    else if(_color == "B" && !ui->filter_color_black->isChecked())
    {
        return true;
    }
    else if(_color == "W" && !ui->filter_color_white->isChecked())
    {
        return true;
    }

    else if(_color == "Art" && !ui->filter_color_artifact->isChecked())
    {
        return true;
    }

    else if(_color.contains("Lnd") && !ui->filter_color_land->isChecked())
    {
        return true;
    }
    else if(_color == "Gld" && !ui->filter_color_gold->isChecked())
    {
        return true;
    }
    else if(_color == "Gld" && ui->filter_color_gold->isChecked())
    {
        if(ui->filter_multicolored->isChecked())
        {
            return checkFilterMulticolor(_card_index);
        }
        else
        {
			return false;
        }
    }
    return false;
}

bool MainWindow::checkFilterMulticolor(int _card_index)
{
    if(cards[_card_index].get_cost().contains(ManaSymbolBlue) && !ui->filter_color_blue->isChecked())
    {
        return true;
    }
    else if(cards[_card_index].get_cost().contains(ManaSymbolBlack) && !ui->filter_color_black->isChecked())
    {
        return true;
    }
    else if(cards[_card_index].get_cost().contains(ManaSymbolRed) && !ui->filter_color_red->isChecked())
    {
        return true;
    }
    else if(cards[_card_index].get_cost().contains(ManaSymbolGreen) && !ui->filter_color_green->isChecked())
    {
        return true;
    }
    else if(cards[_card_index].get_cost().contains(ManaSymbolWhite) && !ui->filter_color_white->isChecked())
    {
        return true;
    }
    return false;
}

bool MainWindow::checkFilterRarity(const QString &_rarity)
{

    if(_rarity == "U" && !ui->filter_rarety_uncommon->isChecked())
    {
        return true;
    }
    else if(_rarity == "C" && !ui->filter_rarety_common->isChecked())
    {
        return true;
    }
    else if(_rarity == "R" && !ui->filter_rarety_rare->isChecked())
    {
        return true;
    }
    else if(_rarity == "M" && !ui->filter_rarety_mythic->isChecked())
    {
        return true;
    }
    else if(_rarity == "B" && !ui->filter_rarety_bonus->isChecked())
    {
        return true;
    }
    else if(_rarity == "S" && !ui->filter_rarety_special->isChecked())
    {
        return true;
    }

    return false;
}

bool MainWindow::checkFilterType(const QString &_type, const QString& _cost)
{
   if(_type.contains("Creature", Qt::CaseInsensitive) && !ui->filter_creature->isChecked())
   {
       return true;
   }

   if(_type.contains("instant", Qt::CaseInsensitive) && !ui->filter_spell_instant->isChecked())
   {
       return true;
   }

   if(_type.contains("sorcery", Qt::CaseInsensitive) && !ui->filter_spell_sorcery->isChecked())
   {
       return true;
   }

   if(_type.contains("Enchantment", Qt::CaseInsensitive) && !ui->filter_spell_enchament->isChecked())
   {
       return true;
   }

   if(_type.contains("Artifact", Qt::CaseInsensitive) && !ui->filter_artifact->isChecked())
   {
       return true;
   }

   if(_type.contains("Planeswalker", Qt::CaseInsensitive) && !ui->filter_planeswalker->isChecked())
   {
       return true;
   }

   if(this->filterOtherTypes(_type,_cost))
   {
       return true;
   }

   return false;
}

bool MainWindow::filterOtherTypes(const QString &_type, const QString& _cost)
{
    if(_cost.isEmpty() && !_type.contains("Land") && !ui->filter_other_type->isChecked())
    {
        return true;
    }


    if(_type.contains("Card", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        return true;
    }

    if(_type.contains("Conspiracy", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        return true;
    }

    if(_type.contains("Emblem", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        return true;
    }

    if(_type.contains("Hero", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        return true;
    }

    if(_type.contains("Phenomenon", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        return true;
    }

    if(_type.contains("Scheme", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        return true;
    }

    if(_type.contains("Token", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        return true;
    }

    if(_type.contains("Vanguard", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        return true;
    }
    if(_type.contains("Plane", Qt::CaseInsensitive) && !ui->filter_other_type->isChecked())
    {
        if(!_type.contains("Planeswalker"))
            return true;
    }
    if(_type.contains("Summon ") && !ui->filter_other_type->isChecked())
    {
        return true;
    }
    if(_type.contains("Eaturecray — Igpay") && !ui->filter_other_type->isChecked())
    {
        return true;
    }
    if(_type.contains("Autobot Character ") && !ui->filter_other_type->isChecked())
    {
        return true;
    }


    return false;

}

bool MainWindow::checkFilterFormat(int _card_index)
{
    if(ui->filter_format->currentText() == "All")
    {
        return false;
    }
    else if(ui->filter_format->currentText() == "Modern")
    {
        if(cards[_card_index].get_legal_modern() > -1)
        {
            return cards[_card_index].get_legal_modern() == 0;
        }
        return !format_modern.contains(cards[_card_index].get_edition());
    }

    else if(ui->filter_format->currentText() == "Standard")
    {
        if(cards[_card_index].get_legal_standard() > -1)
        {
            return cards[_card_index].get_legal_standard() == 0;
        }
        return !format_standard.contains(cards[_card_index].get_edition());
    }

    else if(ui->filter_format->currentText() == "Commander")
    {
        if(cards[_card_index].get_legal_commander() > -1)
        {
            return cards[_card_index].get_legal_commander() == 0;
        }
        return !format_standard.contains(cards[_card_index].get_edition());
    }

    return false;
}

bool MainWindow::checkFilterPower(int _card_index)
{
    bool filtered = false;
    if(!ui->filter_power_min->text().isEmpty())
    {
        QString number = cards[_card_index].get_power();

        if(number.contains("/"))
        {
            number = number.split("/").at(0);
        }
        bool successFlag;
        int power = number.toInt(&successFlag);

        if(successFlag==true)
        {
            filtered = ui->filter_power_min->text().toInt() > power;
        }
        else
        {
            filtered = true;
        }
    }

    if(!ui->filter_power_max->text().isEmpty())
    {
        QString number = cards[_card_index].get_power();

        if(number.contains("/"))
        {
            number = number.split("/").at(0);
        }
        bool successFlag;
        int power = number.toInt(&successFlag);

        if(successFlag==true)
        {
            filtered |= ui->filter_power_max->text().toInt() < power;
        }
        else
        {
            filtered = true;
        }
    }

    return filtered;

}

bool MainWindow::checkFilterCosts(int _card_index)
{
    bool filtered = false;

    if(!ui->filter_costs_min->text().isEmpty())
    {
        filtered = ui->filter_costs_min->text().toInt() > cards[_card_index].get_converted_cost();
    }

    if(!ui->filter_costs_max->text().isEmpty())
    {
        filtered |= ui->filter_costs_max->text().toInt() < cards[_card_index].get_converted_cost();
    }

    return filtered;
}

bool MainWindow::checkFilterLife(int _card_index)
{
    bool filtered = false;
    if(!ui->filter_life_min->text().isEmpty())
    {
        QString number = cards[_card_index].get_life();

        if(number.contains("/"))
        {
            number = number.split("/").at(0);
        }
        bool successFlag;
        int power = number.toInt(&successFlag);

        if(successFlag==true)
        {
            filtered = ui->filter_life_min->text().toInt() > power;
        }
        else
        {
            filtered = true;
        }
    }

    if(!ui->filter_life_max->text().isEmpty())
    {
        QString number = cards[_card_index].get_life();

        if(number.contains("/"))
        {
            number = number.split("/").at(0);
        }
        bool successFlag;
        int power = number.toInt(&successFlag);

        if(successFlag==true)
        {
            filtered |= ui->filter_life_max->text().toInt() < power;
        }
        else
        {
            filtered = true;
        }
    }

    return filtered;
}


void MainWindow::checkFilters()
{
    renderCards();
    updateCardStatistic();
}

void MainWindow::showCardDescription(int _row, int _col)
{
    if(ui->tableWidget->item(_row,ui_const_col_text) != nullptr )
    {
        ui->card_description_textarea->setText(ui->tableWidget->item(_row,ui_const_col_text)->text());
    }
    else
    {
        qDebug() << "Description is null (not only empty)";
    }
	_col++;// DEBUG REMOVE

}

void MainWindow::handleSearch()
{
    renderCards();
    updateCardStatistic();
}

bool MainWindow::isCardSearchFiltered(int _card_index, const QVector<QString> &_words, const QVector<QString> &_inverse_words)
{
    bool hit = true;
    QVector<bool> hits;

    //return !(cards[_card_index].get_edition() == ui->search1->text()); //edition search hack
    QString type = cards[_card_index].get_types().first();

    for(int i = 0; i < _words.size();i++)
    {
        hits.push_back(false);
    }

    //we need hits goback
    for(int i = 0; i < _words.size();i++)
    {
        hits[i] = (cards[_card_index].get_original_text().contains(_words[i], Qt::CaseInsensitive) || type.contains(_words[i],Qt::CaseInsensitive)  || cards[_card_index].get_name().contains(_words[i],Qt::CaseInsensitive));
    }

    //all hits yes = than invert hit and show card

    for(int i = 0; i < _words.size();i++)
    {
        hit = hit && hits[i];
    }

    hit = !hit;


    if(_inverse_words.empty())
    {
        return hit;
    }

    bool inverse_hit = false;


    for(int i = 0; i < _inverse_words.size();i++)
    {
        inverse_hit = inverse_hit || (cards[_card_index].get_original_text().contains(_inverse_words[i], Qt::CaseInsensitive) || type.contains(_inverse_words[i],Qt::CaseInsensitive)  || cards[_card_index].get_name().contains(_inverse_words[i],Qt::CaseInsensitive));
    }

    return hit || inverse_hit;
}

void MainWindow::exportDeck()
{
    QString fileName = QFileDialog::getSaveFileName(this,tr("Save Deck"), "", tr("Deck Files (*.txt)"));
    QString fileNameCsv = fileName+".csv";
    fileName += ".txt";


    QFile outfile(fileName);
    QFile outfileCsv(fileNameCsv);
    outfile.open(QIODevice::WriteOnly);
    outfileCsv.open(QIODevice::WriteOnly);


    // now that we have a file that allows us to write anything to it,
    // we need an easy way to write out text to it
    QTextStream qout(&outfile);
    QTextStream qoutcsv(&outfileCsv);

    const int rows = ui->decktable->rowCount();
    const int cols = ui->decktable->columnCount();

    /*
    for(int i = 0; i < rows; i++)
    {
        QString name = ui->decktable->item(i,1)->text();
        const Card card = getCardFromName(name);
    }
    */

    for(int i = 0; i < rows; i++)
    {
        QString line;
        for(int a = 0; a < cols; a++)
        {
            line += ui->decktable->item(i,a)->text() + ";";
        }
        line+="\n";

        qout << line.replace(";", "\t").toUtf8();
        qoutcsv << line.toUtf8();

    }


    outfile.close();
    outfileCsv.close();
}

void MainWindow::exportDeckMWS()
{
    QString fileName = QFileDialog::getSaveFileName(this,tr("Save Deck"), "", tr("Deck Files (*.mwdeck)"));
    fileName += fileName.endsWith(".mwdeck") ? "" : ".mwdeck";


    QFile outfile(fileName);
    outfile.open(QIODevice::WriteOnly);

    // now that we have a file that allows us to write anything to it,
    // we need an easy way to write out text to it
    QTextStream qout(&outfile);

    const int rows = ui->decktable->rowCount();
	//const int cols = ui->decktable->columnCount();


    /*
    for(int i = 0; i < rows; i++)
    {
        QString name = ui->decktable->item(i,1)->text();
        const Card card = getCardFromName(name);
    }
    */

    for(int i = 0; i < rows; i++)
    {
        QString line;
        line += "    " + ui->decktable->item(i,0)->text() + " [";
        line += ui->decktable->item(i,8)->text() + "] ";
        line += ui->decktable->item(i,1)->text() + "";
        line+="\n\r";

        qout << line;
    }


    outfile.close();
}

const Card& MainWindow::getCardFromName(const QString& _name)
{
    int index;

	for(uint i = 0; i < cards.size(); i++)
    {
        if(cards[i] == _name)
        {
            {
                index = i;
            }
        }
    }
    return cards[index];
}

void MainWindow::showProxyExport()
{

    QString exportData;

    for(int i = 0; i < ui->decktable->rowCount();i++)
    {
        exportData += ui->decktable->item(i,0)->text() + " " + ui->decktable->item(i,1)->text() + "\n";
    }

    QMessageBox msgBox;
    msgBox.setText("Please copy the text to this sitewebsite and click on print:");
    msgBox.setInformativeText("http://www.mtgpress.net/\n\n" + exportData);
    msgBox.setStandardButtons(QMessageBox::Close);
    msgBox.setDefaultButton(QMessageBox::Close);
	msgBox.exec();
}

void MainWindow::testDeck()
{


    qDebug() << ( QString("cd /home/rohezal/projects/magicnet/build-magicnet-Desktop-Debug; ") + "./magicnet `pwd`/" + lastLoadedDeck + " &") .toStdString().c_str();
    system(( QString("/home/rohezal/projects/magicnet/build-magicnet-Desktop-Debug/magicnet ") + lastLoadedDeck + " &").toStdString().c_str());
}

bool MainWindow::searchFilter(int _card_index)
{
    QVector<QString> words;
    QVector<QString> inverse_words;
    if(!ui->search1->text().isEmpty())
    {
        words.push_back(ui->search1->text());
    }

    if(!ui->search2->text().isEmpty())
    {
        words.push_back(ui->search2->text());
    }

    if(!ui->search3->text().isEmpty())
    {
        words.push_back(ui->search3->text());
    }

    if(!ui->search4->text().isEmpty())
    {
        words.push_back(ui->search4->text());
    }

    if(!ui->search_inverse1->text().isEmpty())
    {
        inverse_words.push_back(ui->search_inverse1->text());
    }

    if(!ui->search_inverse2->text().isEmpty())
    {
        inverse_words.push_back(ui->search_inverse2->text());
    }

    if(words.empty() && inverse_words.empty())
    {
        return false;
    }

    return isCardSearchFiltered(_card_index,words,inverse_words);
}

